/*

  barrier.h: header file to use the barrier functions of the psthreads library. 
  Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*
  barrier.h: header file to use the barrier functions of the psthreads library. 
*/
 
#define LONGPADS_PER_CACHE_LINE 8 /* The cache line of the SPARC is 32 Bytes long and a long 
				     is sizeof(long) = 4 Bytes long, therefore 32/4=8. */
#define INTPADS_PER_CACHE_LINE 8 /*  An int has the same size as a long. */


/* We use padding to the size of the cache line to avoid false sharing. */

/* This is the descriptor of a barrier. */

typedef struct _psthread_barrier {  
  int _pad8[INTPADS_PER_CACHE_LINE];
  volatile int global_sense;
  int _pad9[INTPADS_PER_CACHE_LINE - 1];
  volatile long counter;
  long _pad10[LONGPADS_PER_CACHE_LINE - 1];
  volatile int processors;
  int _pad11[INTPADS_PER_CACHE_LINE - 1];
} psthread_barrier_t;

extern void _ps_barrier_init(psthread_barrier_t *thr_incr_barrier, int processors);
extern void _ps_barrier(psthread_barrier_t *thr_incr_barrier);
extern int psthread_barrier_init(psthread_barrier_t *barrier, int thread_count);
extern int psthread_barrier_wait(psthread_barrier_t *barrier);

extern void psthread_yield(void);
