#define SOLARIS
//#define sparc

#ifndef __PS_MD_H__
#define __PS_MD_H__

#define PS_BEGIN_MACRO	{
#define PS_END_MACRO	}

/*****************************************
 * Platform specifics
 */

#if defined (AIX)

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) _setjmp(env)
#define MD_LONGJMP(env, val) _longjmp(env, val)

#define MD_INIT_CONTEXT(_thread, _sp, _main) \
  PS_BEGIN_MACRO                             \
  if (MD_SETJMP((_thread)->context))         \
    _main();                                 \
  (_thread)->context[3] = (long) (_sp);      \
  PS_END_MACRO

#elif defined (FREEBSD)

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) _setjmp(env)
#define MD_LONGJMP(env, val) _longjmp(env, val)

#define MD_INIT_CONTEXT(_thread, _sp, _main)   \
  PS_BEGIN_MACRO                               \
  (void) MD_SETJMP((_thread)->context);        \
  (_thread)->context[0]._jb[0] = (long) _main; \
  (_thread)->context[0]._jb[2] = (long) (_sp); \
  PS_END_MACRO

#elif defined (HPUX)

#define MD_STACK_GROWS_UP
#define MD_SETJMP(env) _setjmp(env)
#define MD_LONGJMP(env, val) _longjmp(env, val)

#ifndef __LP64__
/* 32-bit mode (ILP32 data model) */
#define MD_INIT_CONTEXT(_thread, _sp, _main)        \
  PS_BEGIN_MACRO                                    \
  if (MD_SETJMP((_thread)->context))                \
    _main();                                        \
  ((long *)((_thread)->context))[1] = (long) (_sp); \
  PS_END_MACRO
#else
/* 64-bit mode (LP64 data model) */
#define MD_STACK_PAD_SIZE 256
/* Last stack frame must be preserved */
#define MD_INIT_CONTEXT(_thread, _sp, _main)                     \
  PS_BEGIN_MACRO                                                 \
  if (MD_SETJMP((_thread)->context))                             \
    _main();                                                     \
  memcpy((char *)(_sp) - MD_STACK_PAD_SIZE,                      \
         ((char **)((_thread)->context))[1] - MD_STACK_PAD_SIZE, \
         MD_STACK_PAD_SIZE);                                     \
  ((long *)((_thread)->context))[1] = (long) (_sp);              \
  PS_END_MACRO
#endif /* !__LP64__ */

#elif defined (IRIX)

#include <sys/syssgi.h>

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) setjmp(env)
#define MD_LONGJMP(env, val) longjmp(env, val)

#define MD_INIT_CONTEXT(_thread, _sp, _main) \
  PS_BEGIN_MACRO                             \
  (void) MD_SETJMP((_thread)->context);      \
  (_thread)->context[JB_SP] = (long) (_sp);  \
  (_thread)->context[JB_PC] = (long) _main;  \
  PS_END_MACRO

#elif defined (LINUX)

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) setjmp(env)
#define MD_LONGJMP(env, val) longjmp(env, val)

#define MD_INIT_CONTEXT(_thread, _sp, _main)            \
  PS_BEGIN_MACRO                                        \
  (void) MD_SETJMP((_thread)->context);                 \
  (_thread)->context[0].__jmpbuf[JB_SP] = (long) (_sp); \
  (_thread)->context[0].__jmpbuf[JB_PC] = (long) _main; \
  PS_END_MACRO

#elif defined (WIN32)
#include <setjmp.h>

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) setjmp(env)
#define MD_LONGJMP(env, val) longjmp(env, val)

#define MD_INIT_CONTEXT(_thread, _sp, _main)            \
  PS_BEGIN_MACRO                                        \
  (void) MD_SETJMP((_thread)->context);                 \
  (_thread)->context[4] = (long) (_sp); \
  (_thread)->context[5] = (long) _main; \
  PS_END_MACRO

#elif defined (OSF1)

#include <signal.h>

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) _setjmp(env)
#define MD_LONGJMP(env, val) _longjmp(env, val)

#define MD_INIT_CONTEXT(_thread, _sp, _main)                         \
  PS_BEGIN_MACRO                                                     \
  if (MD_SETJMP((_thread)->context))                                 \
    _main();                                                         \
  ((struct sigcontext *)((_thread)->context))->sc_sp = (long) (_sp); \
  PS_END_MACRO

#elif defined (SOLARIS)

#include <sys/filio.h>
extern int getpagesize(void);

#define MD_STACK_GROWS_DOWN
#define MD_SETJMP(env) setjmp(env)
#define MD_LONGJMP(env, val) longjmp(env, val)

#if defined(sparc) || defined(__sparc)
#define MD_INIT_CONTEXT(_thread, _sp, _main) \
  PS_BEGIN_MACRO                             \
  (void) MD_SETJMP((_thread)->context);      \
  (_thread)->context[1] = (long) (_sp);      \
  (_thread)->context[2] = (long) _main;      \
  PS_END_MACRO
#elif defined(i386) || defined(__i386)
#define MD_INIT_CONTEXT(_thread, _sp, _main) \
  PS_BEGIN_MACRO                             \
  (void) MD_SETJMP((_thread)->context);      \
  (_thread)->context[4] = (long) (_sp);      \
  (_thread)->context[5] = (long) _main;      \
  PS_END_MACRO
#else
#error Unknown CPU architecture
#endif

#else
#error Unknown OS
#endif /* OS */

#ifndef MD_STACK_PAD_SIZE
#define MD_STACK_PAD_SIZE 128
#endif


#endif /* !__PS_MD_H__ */

