/*

  queue.c: queue functions of the psthreads library. 
  Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*
  queue.c: queue functions of the psthreads library. 
*/

#include <stdio.h>
#include "libpsthreads.h"

/* Initialize a queue. */

void _ps_queue_init(ps_queue_t *q)
{
  _ps_lock_init( &q->qlock );
  _ps_lock( &q->qlock );
  q->head = NULL;
  _ps_unlock( &q->qlock );
}


/* Insert an element at the head of a queue. */

void _ps_queue_insert(ps_queue_t *q, psthread_t *t)
{
  _ps_lock( &q->qlock );
  if (q->head == NULL) /* Queue is empty. */
    {
      t->prev = t;
      t->next = t;
      (q->head) = t;
    }
  else
    {
      t->prev = (q->head)->prev;
      t->next = (q->head);
      ((q->head)->prev)->next = t;
      (q->head)->prev = t;
      (q->head) = t;
    }
  _ps_unlock( &q->qlock );
}

/* Append an element at the tail of a queue. */

void _ps_queue_append(ps_queue_t *q, psthread_t *t)
{
  _ps_lock( &q->qlock );
  if (q->head == NULL) /* Queue is empty. */
    {
      t->prev = t;
      t->next = t;
      (q->head) = t;
    }
  else
    {
      t->prev = (q->head)->prev;
      t->next = (q->head);
      ((q->head)->prev)->next = t;
      (q->head)->prev = t;
    }
  _ps_unlock( &q->qlock );
}


/* Remove an element from the head of a queue. */

psthread_t * _ps_queue_removefirst(ps_queue_t *q)
{
  psthread_t *tmp;
  
  _ps_lock( &q->qlock );
  if (q->head == NULL) /* Queue is empty. */
    tmp = NULL;
  else if ((q->head)->next == (q->head)) /* Queue has only one item (its head). */
    {
      tmp = (q->head);
      q->head = NULL;
    }
  else
    {
      ((q->head)->next)->prev = (q->head)->prev;
      ((q->head)->prev)->next = (q->head)->next;
      tmp = (q->head);
      (q->head) = (q->head)->next;
    }
  _ps_unlock( &q->qlock );
  return tmp;
}


/* Remove a specific element from the queue (independently from position). */

psthread_t * _ps_queue_remove(ps_queue_t *q, psthread_t *t)
{
  psthread_t *tmp;  

  _ps_lock( &q->qlock );
  if (q->head == NULL) /* Queue is empty. */
    tmp = NULL;
  else
    {
      tmp = (q->head);
      while( (tmp != t) && (tmp->next) != (q->head) )
	tmp = tmp->next;
      if(tmp == t)
	{
	  if ((q->head)->next == (q->head)) /* Queue has only one item (its head). */
	    q->head = NULL;
	  else
	    {	      
	      (tmp->next)->prev = tmp->prev;
	      (tmp->prev)->next = tmp->next;
	      if ( tmp == (q->head) ) /* We are removing the head. */
		(q->head) = (q->head)->next;	    
	    }
	}  
      else
	{
	  tmp = NULL; /* The element was not found on the queue. */
	}
      _ps_unlock( &q->qlock );
    }
  return tmp; 
}

