/*

  testlib.c: test the functions of the psthreads library. 
  Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*
  testlib.c: test the psthreads library. 
*/

#include <stdio.h>
#include <stdlib.h>
#include "libpsthreads.h"

volatile int num_of_ults = 0;

void thread_routine(void *arg)
{
  long int i;
  long int j = 0;

  /* Since psthread_self() returns -according to the API- a psthread_t structure 
     (and not a pointer to it), what we print with %d is just an id without a 
     particular meaning.  To test the psthread_self() we call it here. */

  fprintf(stdout, "ULT number %d with descriptor %d started on VP %d.\n", 
	  (int)arg, psthread_self(), _ps_find_vpnum());
  fflush(stdout);

  /* A dummy loop. */
  for ( i=0; i<1000000; i++)
    j++;

  /*
    fprintf(stdout, "ULT number %d with descriptor %d yields.\n", (int)arg, psthread_self());
    fflush(stdout);
    psthread_yield();
  */
  
  num_of_ults++;

  fprintf(stdout, "ULT number %d with descriptor %d exits. j = %lu\n", (int)arg, psthread_self(), j);
  fflush(stdout);
  
  psthread_exit();
}


int main()
{
  int i;
  int klt_num = 4;
  int ult_num = 40;
  psthread_t **ult_descr;

  printf("\n");
  printf("testlib.c: test the functions of the psthreads library. \n");  
  printf("Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>\n");
  printf("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. matmul is licensed under the GNU General Public License. See the file COPYING for details.\n");
  printf("\n");

  if ( (ult_descr = (psthread_t **)calloc(ult_num, sizeof(psthread_t *)))==NULL )
    return -1;
  
  
  fprintf(stdout, "psthread_init returns: %d\n", psthread_init(klt_num));
  fflush(stdout);
  
  for(i=0;i<ult_num;i++){
    fprintf(stdout, "Creating ult %d with psthread_create.\n", i);
    fflush(stdout);
    fprintf(stdout, "psthread_create returns: %d\n", 
	    psthread_create(&ult_descr[i], thread_routine, (void *)i, -1 ));
    fflush(stdout);    
  }
  
  /* So that vp 0 can execute ults too, we must call psthread_yield() or 
     psthread_join().  If we want the execution to start early, we call 
     psthread_yield() before the several calls to psthread_join().  To 
     test the psthread_yield() we call it here. */
  psthread_yield();
  
  for(i=0;i<ult_num;i++){
    fprintf(stdout, "Joining ult %d with psthread_join.\n", i);
    fflush(stdout);
    fprintf(stdout, "psthread_join returns: %d\n", 
	    psthread_join(ult_descr[i]));
    fflush(stdout);    
  }

  fprintf(stdout, "Num_of_ults: %d\n", num_of_ults);
  fflush(stdout);
  
  return 0; 
}
