/*

  testqueue.c: test the queue functions of the psthreads library. 
  Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*
  testqueue.c: test the queue functions of the psthreads library. 
*/

#include <stdio.h>
#include "libpsthreads.h"

void queue_print(ps_queue_t *q)
{
  psthread_t *tmp;
  int i=0;

  _ps_lock( &q->qlock );
  if (q->head == NULL) /* Queue is empty. */
    ;
  else
    {
      tmp = (q->head);
      do {
	fprintf( stdout, "Address of node %d: %p\n", i, tmp );
	fflush(stdout);
	tmp = tmp->next;
	i++;
      }
      while ( (tmp) != (q->head) );
    }
  _ps_unlock( &q->qlock );
}

int main()
{
  ps_queue_t q;
  psthread_t t1, t2, t3, t4;
  psthread_t *t5, *t6, *t7;

  printf("\n");
  printf("testqueue.c: test the queue functions of the psthreads library. \n");  
  printf("Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>\n");
  printf("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. matmul is licensed under the GNU General Public License. See the file COPYING for details.\n");
  printf("\n");

  _ps_queue_init(&q);
  _ps_queue_insert(&q, &t1);
  fprintf(stdout, "t1\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  _ps_queue_insert(&q, &t2);
  fprintf(stdout, "t2, t1\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  _ps_queue_insert(&q, &t3);
  fprintf(stdout, "t3, t2, t1\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  _ps_queue_append(&q, &t4);
  fprintf(stdout, "t3, t2, t1, t4\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  t5 = _ps_queue_removefirst(&q); /* t5 = former t3 */
  fprintf(stdout, "t2, t1, t4\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  fprintf( stdout, "Address of former t3 (now t5): %p\n", t5 );
  fflush(stdout);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  t6 = _ps_queue_remove(&q, &t1); /* t6 = former t1 */
  fprintf(stdout, "t2, t4\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

  fprintf( stdout, "Address of former t1 (now t6): %p\n", t6 );
  fflush(stdout);
  fprintf( stdout, "------------\n" );
  fflush(stdout);

/* Remove the head. */
  t7 = _ps_queue_remove(&q, &t2); /* t7 = former t2 */
  fprintf(stdout, "t4\n");
  queue_print(&q);
  fprintf( stdout, "------------\n" );
  fflush(stdout);
  fprintf( stdout, "Address of former t2 (now t7): %p\n", t7 );
  fflush(stdout);
  
  return 0;
}
