/*

  vpmanipul.c: The part of the psthreads non-preemptive, user-level thread 
  library concerning the manipulation of the virtual processors.
  Copyright (C) 2002 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  These functions of the psthreads library concern the
  manipulation of the virtual processors.

*/



#include <stdio.h>
#include <stdlib.h>
#include "libpsthreads.h"

/* Find on which VP we are being executed. */
int _ps_find_vpnum(void)
{
  int vpnum;
  pthread_t thisklt;
  thisklt = pthread_self();
  for( vpnum=0; vpnum<NUM_OF_VPs; vpnum++ )
    if( pthread_equal(thisklt, _ps_klt[vpnum]) !=  0 )
      return vpnum;
  return -1; /* We should never come here. */
}

/* Execute a thread's start routine with its arguments. */
void _ps_run(void)
{
  /* Find the active thread on the VP we are. */
  int vpnum = _ps_find_vpnum();
  psthread_t *actthr;
  actthr=_ps_active_thread[vpnum];

  (*actthr->start_routine)(actthr->arg);

  /* Exit after the end of the execution. */
  psthread_exit();
}

/* Idle thread function. */
void _ps_idle(void)
{
  
  /* While the application has not ended. */
  while ( _ps_actthr_counter > 0 ){
    _ps_schedule();
  }
  
  /* When the application ends, finish the execution of the 
     vp, by exiting its klt. */
  pthread_exit(0);
}

void _ps_schedule(void)
{
  /* Find the active thread on the VP we are. */
  psthread_t *actthr;
  psthread_t *thr2bexecuted;
  int vpnum = _ps_find_vpnum();
  actthr=_ps_active_thread[vpnum];
  
  if( MD_SETJMP(actthr->context) == 0 ){
    thr2bexecuted = _ps_queue_removefirst( &ready_queue );
    if( thr2bexecuted == NULL )
      thr2bexecuted = _ps_idle_thread[vpnum];
    
    thr2bexecuted->status = RUNNING;
    _ps_active_thread[vpnum] = thr2bexecuted;
    MD_LONGJMP(thr2bexecuted->context, 1);
  }
}

void *_ps_vp_init(void *vpnumber)
{
  psthread_t *dummy_thread;
  int vpnum = (int)vpnumber;

  /* Store the kernel-level thread for this VP. */
  _ps_klt[vpnum]=pthread_self();

  /* Create idle user-level thread for this vp. */
  if( psthread_create(&_ps_idle_thread[vpnum], (void *)_ps_idle, NULL, -1) != 0)
    return (void *)-1; /* Something other than NULL, to indicate failure. */

  /* Create dummy user-level thread corresponding to the code already
     being executed. We need this ult just for the first context-switch. 
     We do not use psthread_create, as we do not want this ult to be 
     appended to the ready-queue. */
  dummy_thread = (psthread_t *)calloc(1, sizeof(psthread_t));
  if ( dummy_thread == NULL )
    return (void *)-1; /* Something other than NULL, to indicate failure. */
  dummy_thread->status = RUNNING;
  _ps_active_thread[vpnum] = dummy_thread;
  
  _ps_lock( &_mutex_counter );
  _ps_actthr_counter++;
  _ps_unlock( &_mutex_counter );

  /* Execute scheduling code. */
  _ps_schedule();
  
  return NULL; /* Success. */
}	

